import { SubscribeMessage, WebSocketGateway } from '@nestjs/websockets';
import { Socket } from 'socket.io';
import { ResultDTO } from 'src/result.dto';
import { AuthService } from './auth.service';
import { UserService } from '../user/user.service';

@WebSocketGateway({ namespace: 'api' })

export class AuthGateway {

    constructor(
        private _auth: AuthService,
        private _user: UserService) {
        console.log('Auth gateway readed.');
    }
    @SubscribeMessage('authLogin')
    authLogin(client: Socket, data: any): Promise<ResultDTO> {
        console.log('authLogin',data);
        return new Promise((resolve, reject) =>{
            this._user.login(data).then(user=>{
                const result=new ResultDTO();
                console.log("user:",user);
                if(user){
                    result.ok=1;
                    result.msg= "登录许可已颁发."
                    //登录成功 传入_id 生成一个hash 返回. 用作自动登录
                    this._auth.create(user._id,7).then(hash=>{
                        result.hash=hash.key;
                        console.log("result:",result);
                        resolve(result);
                    });
                    
                    
                }else{
                    result.msg='用户名或密码错误!';
                    result.ok=-1;
                    resolve(result)
                }
            })
            
        })
    }

    @SubscribeMessage('auth')
    auth(client: Socket, data: any): Promise<ResultDTO> {
        // console.log('checkAuth', data);
        return new Promise((resolve) => {
            this._auth.authModel.findOne({ key: data.auth }).then(auth => {
                const result = new ResultDTO();
                if (auth) {
                    this._user.userModel.findById(auth.userId).then(user => {
                        result.data = user;
                        result.ok = 1;
                        //设置当前用户
                        client.request.user = user;
                        result.msg = "许可通过!";
                        console.log('许可通过:', data,user);
                        resolve(result);
                    })
                    //登录成功 传入_id 生成一个hash 返回. 用作自动登录

                } else {
                    result.msg = '无效许可!';
                    console.log('许可无效:', data);
                    result.ok = -1;
                    resolve(result)
                }


            })



        })


    }


    @SubscribeMessage('authCheck')
    authCheck(client: Socket, data: any): Promise<ResultDTO> {
        return new Promise((resolve) => {
            const result = new ResultDTO();
            result.ok = 1;
            //判断当前用户等级
            if (client.request.user.level < 3) {
                result.data = true;
            } else {
                result.data = false;
            }

            resolve(result)
        })
    }


}
