import { Item } from './item.model';
import { Orders } from '../orders/orders.model';
import { OrdersService } from '../orders/orders.service';
import { SubscribeMessage, WebSocketGateway } from '@nestjs/websockets';
import { Socket,Client,Server} from 'socket.io';
import { ResultDTO } from 'src/result.dto';
import { UserService } from '../user/user.service';
import { ItemService } from './item.service';
import { SetMetadata, UseGuards } from '@nestjs/common';
import { AuthGuard } from 'src/guard/auth.guard';
import { Role } from 'src/_enum/role.enum';
import { Roles } from 'src/_decorator/roles.decorator';


@WebSocketGateway({namespace: 'api'})
export class ItemGateway {
    
    constructor(
        private _user:UserService,
        private _od:ItemService,
        private _o:OrdersService
        ) {
        console.log('Item gateway readed.');
    }


    @UseGuards(AuthGuard)
    @SubscribeMessage('ItemGet')
    @Roles(Role.Admin)
    //@UseGuards(AuthGuard)
    Get( client: Socket, filter: any): Promise<ResultDTO> {
        
         this._user.findOne({}).then(doc=>{
            console.log(doc);
         })
        console.log('ItemGet',filter);
        return new Promise((resolve, reject) =>{
            const result=new ResultDTO();
            this._od.Get(filter).then(docs=>{
                console.log("Finded:",docs.length);
                if(docs){
                    result.ok=1;
                    result.msg= "Geted Item."
                    result.count=docs.length;
                    result.data=docs
                    resolve(result)
                }
            })
        }) 
    }

    @SubscribeMessage('ItemGetOne')
    GetOne( client: Socket, filter: any): Promise<ResultDTO> {
        console.log('ItemGetOne',filter);
        return new Promise((resolve) =>{
            const result=new ResultDTO();
            this._od.GetOne(filter).then(doc=>{
                if(doc){
                    result.ok=1;
                    result.msg= "Geted."
                    result.count=1;
                    result.data=doc
                    resolve(result)
                }else{
                    result.ok=0;
                    result.msg= "Not data."
                    result.data=null;
                    resolve(result)
                }
            })
        })
    }

    @SubscribeMessage('ItemAdd')
    Add(client: Socket, data: any): Promise<ResultDTO> {
        console.log('ItemAdd',data);
        return new Promise((resolve, reject) =>{
            let result=new ResultDTO();
            this._od.Add(data).then(doc=>{
                if(doc){
                    //添加数据完成 统计数据更新表单
                   
                    result.ok=1;
                    result.msg= "Added."
                    result.count=1;
                    result.data=doc
                    resolve(result)
                }
                
        })
        })

        
    }

    
    @SubscribeMessage('ItemUpd')
    Update(client: Socket, data: any): Promise<ResultDTO> {
        return new Promise((resolve, reject)=>{
            const result=new ResultDTO();
            result.ok=1;
            //判断当前用户等级
            this._od.Update(data).then(doc=>{
                console.log('UPD',doc)
                result.msg='Upded Item'
                result.data=doc;
                resolve(result)
            }).catch(err=>{
                result.ok=0;
                result.msg=err.message;
                resolve(result)
            })
           
        })
    }

    @SubscribeMessage('ItemDel')
    Delete(client: Socket, data: any): Promise<ResultDTO> {
        //判断当前用户等级
        return new Promise((resolve, reject)=>{
            console.log("删除",data);
            const result=new ResultDTO();
            this._od.GetOne({'_id':data}).then(doc=>{
                console.log('查获',doc);
                if(doc){
                    this._od.Delete(doc._id).then(ret=>{
                        console.log('del ret',ret);
                        if(ret.deletedCount>0){
                            result.ok=1;
                            result.msg=`已删除数据: ${ret.deletedCount}`;
                            resolve(result);
                        }else{
                            console.log("无可删除数据");
                            result.ok=0;
                            result.msg="无可删除数据."
                            resolve(result);
                        }
                        
                    })
                }else{
                    console.log("无可删除数据");
                    result.ok=0;
                    result.msg="无可删除数据."
                    resolve(result);
                }
                
            }).catch(error=>{
                result.ok=0;
                result.msg=error.message;
                reject(result)
            })
        })
    }
    
}
