import { Orders } from './../orders/orders.model';
import { SubscribeMessage, WebSocketGateway } from '@nestjs/websockets';
import { Socket,Client,Server} from 'socket.io';
import { ResultDTO } from 'src/result.dto';
import { UserService } from '../user/user.service';
import { PaymentService } from './payment.service';
import { UseGuards } from '@nestjs/common';
import { AuthGuard } from 'src/guard/auth.guard';
import { AuthModule } from '../auth/auth.module';
import { OrdersService } from '../orders/orders.service';


@WebSocketGateway({namespace: 'api'})

export class PaymentGateway {
    
    constructor(private _user:UserService,private _payment:PaymentService,private _o:OrdersService) {
        console.log('payment gateway readed.');
    }

    reTotalOrder(_id): Promise<Orders>{
        return new Promise((resolve, reject) =>{
            this._payment.Get({"orders":_id}).then(payList=>{
                let totalPayment=0;
                //遍历统计
                payList.map((i)=>{
                    totalPayment+= i.payment;
                })
                //查找更新 发布
                this._o.GetOne({'_id':_id}).then(o=>{
                    o.payment=totalPayment;
                    o.balance=o.total-totalPayment;
                    o.save().then(ret=>{
                        resolve(ret)
                        //client.emit('freshOrder',ret)
                    })
                })
            })
        })
       
    }

    @SubscribeMessage('paymentGet')
    //@UseGuards(AuthGuard)
    Get( client: Socket, filter: any): Promise<ResultDTO> {
        console.log('E paymentGet',filter);
        return new Promise((resolve, reject) =>{
            let result=new ResultDTO();
            this._payment.Get(filter).then(docs=>{
                console.log("F paymentGet:",docs.length);
                if(docs){
                    result.ok=1;
                    result.msg= "Geted.Order"
                    result.count=docs.length;
                    result.data=docs
                    resolve(result)
                    //client.emit("test",{'aa':'11111'})
                }
            })
        }) 
    }

    @SubscribeMessage('paymentGetOne')
    GetOne( client: Socket, filter: any): Promise<ResultDTO> {
        console.log('paymentGetOne',filter);
        return new Promise((resolve, reject) =>{
            let result=new ResultDTO();
            this._payment.GetOne(filter).then(doc=>{
                if(doc){
                    result.ok=1;
                    result.msg= "Geted."
                    result.count=1;
                    result.data=doc
                    resolve(result)
                }
            })
        })
    }

    @SubscribeMessage('paymentAdd')
    Add(client: Socket, data: any): Promise<ResultDTO> {
        console.log('paymentAdd',data);
        return new Promise((resolve, reject) =>{
            let result=new ResultDTO();
            this._payment.Add(data).then(doc=>{
                if(doc){
                    this.reTotalOrder(doc.orders).then(order=>{
                        client.emit('freshOrder',order)
                    })
                    result.ok=1;
                    result.msg= "Added."
                    result.count=1;
                    result.data=doc
                    resolve(result)
                    // this._payment.GetOne(doc).then(_data=>{
                    //     result.ok=1;
                    //     result.msg= "Added."
                    //     result.count=1;
                    //     result.data=_data
                    //     resolve(result)
                    // })
                    
                }
                
        })
        })

        
    }

    
    @SubscribeMessage('paymentUpd')
    Update(client: Socket, data: any): Promise<ResultDTO> {
        return new Promise((resolve, reject)=>{
            let result=new ResultDTO();
            result.ok=1;
            //判断当前用户等级
            this._payment.Update(data).then(doc=>{
                this.reTotalOrder(doc.orders).then(order=>{
                    client.emit('freshOrder',order)
                })
                console.log('UPD',doc)
                result.msg='Upded'
                result.data=doc;
                resolve(result)
            })
           
        })
    }

    @SubscribeMessage('paymentDelete')
    Delete(client: Socket, data: any): Promise<ResultDTO> {
        return new Promise((resolve, reject)=>{

            console.log("删除",data);
            let result=new ResultDTO();
            this._payment.GetOne({'_id':data}).then(doc=>{
                console.log('查获',doc);
                if(doc){
                    this._payment.Delete(doc._id).then(ret=>{
                        console.log('del ret',ret);
                        if(ret.deletedCount>0){
                            result.ok=1;
                            result.msg=`已删除数据: ${ret.deletedCount}`;
                            this.reTotalOrder(doc.orders).then(order=>{
                                client.emit('freshOrder',order)
                            })
                            resolve(result);
                        }else{
                            console.log("无可删除数据");
                            result.ok=0;
                            result.msg="无可删除数据."
                            resolve(result);
                        }
                        
                    })
                }else{
                    console.log("无可删除数据");
                    result.ok=0;
                    result.msg="无可删除数据."
                    resolve(result);
                }
                
            }).catch(error=>{
                result.ok=0;
                result.msg=error.message;
                reject(result)
            })
        })
    }
    
}
